﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
   public partial class FormCocktails : Form
    {
        private readonly ILogger _logger;
        private readonly ICocktailLogic _logic;

        public FormCocktails(ILogger<FormCocktails> Logger, ICocktailLogic Logic)
        {
            InitializeComponent();

            _logger = Logger;
            _logic = Logic;
        }

        private void FormCocktails_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var List = _logic.ReadList(null);

                if (List != null)
                {
                    DataGridView.DataSource = List;
                    DataGridView.Columns["Id"].Visible = false;
                    DataGridView.Columns["CocktailName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                    DataGridView.Columns["CocktailComponents"].Visible = false;
                }

                _logger.LogInformation("Загрузка коктеля");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки коктеля");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormCocktail));

            if (Service is FormCocktail Form)
            {
                if (Form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                var Service = Program.ServiceProvider?.GetService(typeof(FormCocktail));

                if (Service is FormCocktail Form)
                {
                    var Temp = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    Form.Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);

                    if (Form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Удаление коктеля");
                    try
                    {
                        if (!_logic.Delete(new CocktailBindingModel
                        {
                            Id = id
                        }))
                        {
                            throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
                        }

                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка удаления коктеля");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void DataGridView_SelectionChanged(object sender, EventArgs e)
        {
        }
    }
}
