using BarBusinessLogic.BusinessLogics;
using BarBusinessLogic.OfficePackage.Implements;
using BarBusinessLogic.OfficePackage;
using BarContracts.BusinessLogicContracts;
using BarContracts.StoragesContracts;
using BarDatabaseImplement.Implements;
using BarView.Forms;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using BarContracts.BusinessLogicsContracts;
using BarBusinessLogic.MailWorker;
using BarContracts.BindingModels;

namespace BarView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;

        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();

            var Services = new ServiceCollection();
            ConfigureServices(Services);

            _serviceProvider = Services.BuildServiceProvider();

            try
            {
                var MailSender = _serviceProvider.GetService<AbstractMailWorker>();
                MailSender?.MailConfig(new MailConfigBindingModel
                {
                    MailLogin = System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ?? string.Empty,
                    MailPassword = System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ?? string.Empty,
                    SmtpClientHost = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ?? string.Empty,
                    SmtpClientPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
                    PopHost = System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
                    PopPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
                });

                var Timer = new System.Threading.Timer(new TimerCallback(MailCheck!), null, 0, 5000);
            }
            catch (Exception ex)
            {
                var Logger = _serviceProvider.GetService<ILogger>();
                Logger?.LogError(ex, "   ");
            }

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void MailCheck(object obj) => ServiceProvider?.GetService<AbstractMailWorker>()?.CheckMailAsync();

        private static void ConfigureServices(ServiceCollection Services)
        {
            Services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });

            Services.AddTransient<IComponentStorage, ComponentStorage>();
            Services.AddTransient<IOrderStorage, OrderStorage>();
            Services.AddTransient<ICocktailStorage, CocktailStorage>();
            Services.AddTransient<IClientStorage, ClientStorage>();
            Services.AddTransient<IImplementerStorage, ImplementerStorage>();
            Services.AddTransient<IMessageInfoStorage, MessageInfoStorage>();

            Services.AddTransient<IComponentLogic, ComponentLogic>();
            Services.AddTransient<IOrderLogic, OrderLogic>();
            Services.AddTransient<ICocktailLogic, CocktailLogic>();
            Services.AddTransient<IReportLogic, ReportLogic>();
            Services.AddTransient<IClientLogic, ClientLogic>();
            Services.AddTransient<IImplementerLogic, ImplementerLogic>();
            Services.AddTransient<IWorkProcess, WorkModeling>();
            Services.AddTransient<IMessageInfoLogic, MessageInfoLogic>();

            Services.AddTransient<AbstractSaveToWord, SaveToWord>();
            Services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            Services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
            Services.AddSingleton<AbstractMailWorker, MailKitWorker>();

            Services.AddTransient<FormMain>();
            Services.AddTransient<FormComponent>();
            Services.AddTransient<FormComponents>();
            Services.AddTransient<FormCreateOrder>();
            Services.AddTransient<FormCocktail>();
            Services.AddTransient<FormCocktailComponent>();
            Services.AddTransient<FormCocktails>();
            Services.AddTransient<FormReportCocktailComponents>();
            Services.AddTransient<FormReportOrders>();
            Services.AddTransient<FormClients>();
            Services.AddTransient<FormImplementers>();
            Services.AddTransient<FormImplementer>();
            Services.AddTransient<FormMail>();
        }
    }
}
